#ifndef DISPATCHERS_H
#define DISPATCHERS_H



#include <functional>

typedef std::function<void(const std::string &)> StringFunc;
typedef std::function<void(void)> notifyFunc;

/*
Item of the day:
class Result {
    // holds unique_ptr or shared_ptr to the "dispatcher"
    // when I'm done -- delete the result, cleaning up
    // the dispatcher

}

 How does this call another?  
 How does it return a result?

possibly:  io_service::post( DONE );  !

 */


class Session;


class Dispatch {
protected:
  Session *sess;
  notifyFunc notify_;
public:
  Dispatch(Session *);
  virtual ~Dispatch();

  void setNotify(notifyFunc nf);
  void notify(void);

  const std::string &get_prompt(void);
  void to_server(const std::string &send);
  void to_client(const std::string &send);
};

class MainDispatch : public Dispatch {
public:
  MainDispatch(Session*);

  void activate(void);
  void deactivate(void);

  void server_line(const std::string &line);
  void server_prompt(const std::string &prompt);
  void client_input(const std::string &input);
private:
  int count;  
  std::string old_prompt;
};

class CoreDispatch : public Dispatch {
public:
  CoreDispatch(Session *);


  void server_line(const std::string &line);
  void server_prompt(const std::string &prompt);
  void client_input(const std::string &input);
};

#include "session.h"

#endif
